import java.io.*;

public class Code_Vigenere
{
	final static double	ci_german = 0.0762;
	final static double	ci_english = 0.0661;
	final static double	ci_random = 0.0385;
	
	final static char	LETTERS = 26;
	
	Code_Vigenere()
	{
	}
	
	//##########################################################################
	int	gcd(int z1 , int z2)
	{
		int	z3;

		if ((z1 == 0) || (z2 == 0))
			return 0;

		if (z1 < z2)
		{
			z3=z1;
			z1=z2;
			z2=z3;
		}
		
		do
		{
			z3 = z1 % z2;
			z1 = z2;
			z2 = z3;
		}
		while (z3 != 0);

		return z1;
	}

	//##########################################################################
	String	ReadFile(String fname)
	{
		String	result = "";
		BufferedReader	in = null;
		try
		{
			in = new BufferedReader(
  							new InputStreamReader(
    						new FileInputStream( fname ) ) );
			
			String	s = "";
			do
			{
				s = in.readLine();
				if (s != null)
					result += s+"\n";
			}
			while (s != null);
		}
		catch (IOException e)
		{
			result += e.getMessage()+"\n";
		}

		try
		{
			in.close();
		}
		catch (IOException e)
		{
			result += e.getMessage()+"\n";
		}

		//reduce text to uppercase-letters
		result = result.trim();
		result = result.toUpperCase();
		for (int i=0; i<result.length(); i++)
		{
			if ((result.charAt(i) < 'A') || (result.charAt(i) > 'Z'))
				result = result.substring(0,i).trim()+result.substring(i+1,result.length()).trim();
		}
		
		return result;
	}



	//##########################################################################
	int	GetKeyLength_Friedmann(String s)
	{
		double	n,len;
		int	n2,len2;

		n2 = s.length();
		n = n2;
		len = (0.0377*n)/(ci_german*(n-1)-0.0385*n+0.0762);
		len2 = (int)Math.round(len);

		String s2 = "Lnge der Nachricht : "+Integer.toString(n2);
//		MessageBox	msg = new MessageBox(this , "Friedmann" , s2);
		return len2;
	}




	//##########################################################################
	int	GetKeyLength_Kasiski(String s)
	{
		String	subs[] = new String[s.length()];
		int		count[] = new int[s.length()];
		int		CountBlocks = 0;

		int		indizes[] = new int[s.length()];
		int		dist[] = new int[s.length()];
		int		CountDist = 0;
		int		CountDistTemp;

		String	str;
		int		len = 0;
		int		block = 3;
		int		i,j,k,index,index2,cnt;
		int		alreadyfound;
		
		for (i=0; i<s.length()-block; i++)
		{
			str = s.substring(i,i+block);
			index = -1;
			cnt = -1;
			CountDistTemp = CountDist;
			do
			{
				index2 = s.indexOf(str , index+1);
				
				if ((index>-1) && (index2 != -1))
				{
					dist[CountDistTemp] = index2-index;
					CountDistTemp++;
				}
				index = index2;
				cnt++;
			}
			while (index >= 0);

			if (cnt > 1)
			{
				//check if substring already occured in the ciphertext
				alreadyfound = -1;
				for (j=0; j<CountBlocks; j++)
				{
					if (subs[j].equals(str) == true)
						alreadyfound = j;
				}
					
				if (alreadyfound == -1)
				{
					//if not, then save the distance to the predecessor
					CountDist = CountDistTemp;
					
					subs[CountBlocks] = str;
					count[CountBlocks] = cnt;
					CountBlocks++;
				}
			}
		}


		int	gcds[] = new int[CountDist*CountDist];
		//calculate the gcd's between all possible combinations of the distances
		cnt = 0;
		for (i=0; i<CountDist; i++)
		{
			for (j=0; j<CountDist; j++)
			{
				if ((i != j) && (gcd(dist[i],dist[j]) != 1))
				{
					k = gcd(dist[i],dist[j]);
					alreadyfound = -1;
					for (index=0; index<cnt; index++)
					{
						if (indizes[index] == k)
						{
							alreadyfound = index;
							count[index]++;
						}
					}
					if (alreadyfound == -1)
					{
						indizes[cnt] = k;
						count[cnt] = 1;
						cnt++;
					}
				}
			}
		}
		//sort gcd's by frequency
		for (i=0; i<cnt-1; i++)
		{
			int	maxpos = i;
			int tmp;
			for (j=i+1; j<cnt; j++)
			{
				if (count[j] > count[maxpos])
					maxpos = j;
			}
			
			if (maxpos != i)
			{
				tmp = count[i];
				count[i] = count[maxpos];
				count[maxpos] = tmp;
				tmp = indizes[i];
				indizes[i] = indizes[maxpos];
				indizes[maxpos] = tmp;
			}
		}
/*		//in indizes[] the possible keylength's are stored by decreasing possibility
		String	s2 = "Geschtzte Schlssellnge : "+Integer.toString(indizes[0]);
		s2 += "\n(Nchstwahrscheinlichere Schlssellngen :";
		if (cnt > 0)
			s2 += "\n"+Integer.toString(indizes[1]);
		if (cnt > 1)
			s2 += "\n"+Integer.toString(indizes[2]);
		if (cnt > 2)
			s2 += "\n"+Integer.toString(indizes[3]);
		MessageBox	msg = new MessageBox(this , "Kasiski" , s2);
*/
		return indizes[0];
	}



	//##########################################################################
	String	GetKey(String str , int keylen)
	{
		String	key = "";
		String	Spalten[] = new String[keylen];
		int		i,j,k,inDictionary;
		int		CountLetters;

		//TODO : make global final static
		double	LetterFrequencyGerman[] = new double[LETTERS];
		char	LookupLetterGerman[] = new char[LETTERS];

		double	This_Letter_Freq[] = new double[LETTERS];
		char	This_Lookup_Letter[] = new char[LETTERS];

		if ((str.length() <= 0) || (keylen <= 0) || (keylen > str.length()))
			return "";

		
		for (i=0; i<keylen; i++)
			Spalten[i] = "";

		//build monoalphabetic columns
		for (i=0; i<str.length(); i++)
			Spalten[i%keylen] += str.charAt(i);

		//initialize Letter-Frequency of german language (Source : WIKIPEDIA)
		LookupLetterGerman[0]='E';
		LookupLetterGerman[1]='N';
		LookupLetterGerman[2]='I';
		LookupLetterGerman[3]='S';
		LookupLetterGerman[4]='R';
		LookupLetterGerman[5]='A';
		LookupLetterGerman[6]='T';
		LookupLetterGerman[7]='D';
		LookupLetterGerman[8]='H';
		LookupLetterGerman[9]='U';
		LookupLetterGerman[10]='L';
		LookupLetterGerman[11]='C';
		LookupLetterGerman[12]='G';
		LookupLetterGerman[13]='M';
		LookupLetterGerman[14]='O';
		LookupLetterGerman[15]='B';
		LookupLetterGerman[16]='W';
		LookupLetterGerman[17]='F';
		LookupLetterGerman[18]='K';
		LookupLetterGerman[19]='Z';
		LookupLetterGerman[20]='P';
		LookupLetterGerman[21]='V';
		LookupLetterGerman[22]='J';
		LookupLetterGerman[23]='Y';
		LookupLetterGerman[24]='X';
		LookupLetterGerman[25]='Q';


		LetterFrequencyGerman[0]=17.40;
		LetterFrequencyGerman[1]=9.78;
		LetterFrequencyGerman[2]=7.55;
		LetterFrequencyGerman[3]=7.27;
		LetterFrequencyGerman[4]=7.00;
		LetterFrequencyGerman[5]=6.51;
		LetterFrequencyGerman[6]=6.15;
		LetterFrequencyGerman[7]=5.08;
		LetterFrequencyGerman[8]=4.76;
		LetterFrequencyGerman[9]=4.35;
		LetterFrequencyGerman[10]=3.44;
		LetterFrequencyGerman[11]=3.06;
		LetterFrequencyGerman[12]=3.01;
		LetterFrequencyGerman[13]=2.53;
		LetterFrequencyGerman[14]=2.51;
		LetterFrequencyGerman[15]=1.89;
		LetterFrequencyGerman[16]=1.89;
		LetterFrequencyGerman[17]=1.66;
		LetterFrequencyGerman[18]=1.21;
		LetterFrequencyGerman[19]=1.13;
		LetterFrequencyGerman[20]=0.79;
		LetterFrequencyGerman[21]=0.67;
		LetterFrequencyGerman[22]=0.27;
		LetterFrequencyGerman[23]=0.04;
		LetterFrequencyGerman[24]=0.03;
		LetterFrequencyGerman[25]=0.02;



		//build letter-frequency-table for each column
		//(aim : get the most frequent letter and assume that this is an 'E' in plaintext)
		for (i=0; i<keylen; i++)
		{
			CountLetters = 0;
			for (j=0; j<LETTERS; j++)
			{
				This_Letter_Freq[j] = 0;
				This_Lookup_Letter[j] = 0;
			}
			
			for (j=0; j<Spalten[i].length(); j++)
			{
				//check whether the letter is already in the dictionary
				inDictionary = -1;
				for (k=0; k<CountLetters; k++)
				{
					if (This_Lookup_Letter[k] == Spalten[i].charAt(j))
						inDictionary = k;
				}
				//if letter is not in dictionary then create new entry , else increase counter
				if (inDictionary == -1)
				{
					This_Lookup_Letter[CountLetters] = Spalten[i].charAt(j);
					This_Letter_Freq[CountLetters] = 1;
					CountLetters++;
				}
				//Letter already in dictionary ?
				else
					This_Letter_Freq[inDictionary] += 1;
			}//for (j=0; j<SubString[i].length(); j++)
		
			//diese sortieren	(BubbleSort)
			for (j=0; j<CountLetters-1; j++)
			{
				int		maxpos = j;
				char	tmp_c;
				double	tmp_d;
				
				for (k=j+1; k<CountLetters; k++)
				{
					if (This_Letter_Freq[k] > This_Letter_Freq[maxpos])
						maxpos = k;
				}
			
				if (maxpos != j)
				{
					tmp_d = This_Letter_Freq[j];
					This_Letter_Freq[j] = This_Letter_Freq[maxpos];
					This_Letter_Freq[maxpos] = tmp_d;
					tmp_c = This_Lookup_Letter[j];
					This_Lookup_Letter[j] = This_Lookup_Letter[maxpos];
					This_Lookup_Letter[maxpos] = tmp_c;
				}
			}

			for (j=0; j<CountLetters; j++)
				This_Letter_Freq[j] /= Spalten[i].length();

			//assume that the most frequent letter is an 'E'
			char	c;
			if (This_Lookup_Letter[0] > LookupLetterGerman[0])
				c = (char)(This_Lookup_Letter[0]-LookupLetterGerman[0]);
			else
				c = (char)('Z'-This_Lookup_Letter[0]+LookupLetterGerman[0]-'A');
			c += 'A';
			key += c;



		}//for (i=0; i<keylen; i++)
		return key;
	}



	//##########################################################################
	String	Decode(String s , String key)
	{
		char	c;
		String	result = "";
		for (int i=0; i<s.length(); i++)
		{
			c = (char)(s.charAt(i)-(key.charAt(i % key.length())-'A') );
			if (c < 'A')
				c = (char)('Z'-('A'-c-1));
			result += c;
		}
		return result;
	}
	
};